function [Sh, Sn, F0] = hn_sep(S, fs, param)
%%%%% Decomposition of the speech signal S into harmonic components Sh and
%%%%% aperiodic components Sn.
%
% Input arguments;
% S: speech signal to decompose
% fs: sampling frequency
% param: set of parameters
%
% Output arguments:
%
% Sh: estimated periodic components
% Sn: estimated noisy components
% F0: estimated pitch. Structure with field F0.t is the time vector, in s,
% and F0.f is the corresponding pitch, in Hz
%
% B. Elie and G. Chardon, last update: december 2016

% S must be a column vector
S = S(:);
F0 = struct('f',[],'t',[]);

Lsec = 0.04; % default length of the window, in s
Lmedhz = 1000; % default length of the median filtering, in Hz.

%%%%% Default values of param
if nargin < 3; param = []; end;
if ~isfield(param,'lw'); param.lw = 2^nextpow2(Lsec * fs); end;
if ~isfield(param,'thrs'); param.thrs = 2; end;
if ~isfield(param,'Nfft'); param.nfft = param.lw*4; end;
if ~isfield(param,'lfmed'); param.lfmed = ceil(Lmedhz/fs * param.nfft); end;
if ~isfield(param,'nh'); param.nh = 1000; end;

Lw = param.lw;
Nfft = param.nfft;
Lmed = param.lfmed;
Npart = param.nh;
Thres = param.thrs;

%%%% Build sliding window for the time signal
H = hann(Lw);
H = H(1:Lw);
H = H/norm(H);
a = Lw/4;

%%%% zero-padding of the signal at both extremities
Spad = [zeros(Lw, 1) ; S ; zeros(2*Lw, 1)];

% Initialization
Sproj = zeros(size(Spad));
f0 = NaN(ceil((length(Spad)-2*Lw)/a),1);
t = (0:ceil((length(Spad)-2*Lw)/a)-1)*(a/fs);%+(Lw/2/fs);

%%% For each time frame
for u = 1:ceil((length(Spad)-2*Lw)/a)
    s = Spad(u*a:u*a+Lw-1).*H; % windowing  
    [ f0(u), APart] = f0harm(s, Nfft, Lmed, Thres, fs, Npart); % Get pitch and detected harmonics
    if ~isempty(APart) % If voicing is present
        E = exp(1i * f0(u)/fs*(1:Lw)'*2*pi * [-APart APart]).* repmat(H, 1, length(APart)*2);% Subspace spanned by the detected harmonics
        E = E./ repmat(sqrt(sum(abs(E).^2, 1)), size(E, 1), 1);
        coeffs = E'*s;% .* [flipud(wiener) ; wiener];   Projection    
        Sproj(u*a:u*a+Lw-1) = Sproj(u*a:u*a+Lw-1) + real((E*coeffs)).*H; % Reconstruction
    end    
end
F0.f = f0;
F0.t = t;

%%%%
A = 1/ (max(H).^2 * 3/2);
Sh = Sproj * A; %%%% Compute the periodic component

Sh = Sh(Lw+1:Lw+length(S));
Sn = S - Sh; %%%% Substract sh to get the aperiodic component

end

function [f0, APart] = f0harm(S, Nfft, Lmed, Thres, fs, NPart)
% Estimation of the fundamental frequency of the windowed signal S

% Input arguments:
%
% S: signal
% Nfft: length of the FFT
% Lmed: length of the median filter
% Thres: threshold for the partial detection
% fs: sampling frequency
% Npart: max number of partials

%%% Set max frequency
Fmax = 4e3;%fs/2;

% Compute the periodogram
P = abs(fft(S, Nfft).^2);
omega = linspace(0, fs-1, Nfft); 
Pw = fftshift(medfilt1(fftshift(P),Lmed)); % Whitened periodogram, median filtering

Pt = P;
% Boundaries of the fundamental frequency
F0min = 80;
F0max = 600;

% Remove periodogram which is under the mean noise level enhanced by threshold Thresh
Pt((P./Pw) < Thres) = 0;
% Remove the periodogram outside Fmax and under lower boundary of the pitch estimation
Pt(omega > Fmax) = 0;
Pt(omega < F0min) = 0;
idx = local_maxes(Pt); % find local max
Pp = zeros(size(P));
w = 3;

% Keep w sampling around local max
for u = 1:length(idx)
    imin = max(1,idx(u)-w);
    imax = min(idx(u)+w, length(P));
    Pp(imin:imax) = P(imin:imax);
end
Ph = zeros(size(P));
Rh = zeros(size(P));

%%% Compute cumulative periodogram
idx0 = find(omega > F0min & omega < F0max);
for u = idx0
    fonda = 0;
    np = 1;
    while omega(u) * np < Fmax
        p = Pp((u-1)*np+1);
        if p > 0
            Rh(u) = Rh(u) + np;
            if np == 1
                fonda = 1;
            end
        end
        Ph(u) = Ph(u) + p;
        np = np + 1;
    end
    if fonda == 0
        Ph(u) = 0;
    end
end

m = max(Ph);
alpha = 0.95;

% Find the max of the cumulative periodogram
idxpot = find(Ph > m * alpha);
[~, idxpotminh] = min(Rh(idxpot));
idxf0 = idxpot(idxpotminh); 

% If empty, no voicing, the signal contains only noise
if isempty(idxf0)
    f0 = 0;
    APart = [];
    return
end

%%% Get the detected harmonics
f0 = omega(idxf0);
APart = [];
nhmax = floor((Fmax)/f0);
NPart = min(NPart,nhmax);
voiced = 1;
for np = 1:NPart
    p = Pt((idxf0-1)*np+1);
    if (any(np==1:3) && p <= 0); voiced=0; end;
    if (p > 0 && voiced )% && ids2 <= 0.1*nidx )
%         if ( ids2 <= 0.2*nidx && voiced )
        APart = [APart np];
 	end
end

if (length(APart) <= 2 || APart(1) ~= 1 || APart(2) ~= 2 || ~voiced)
    f0 = 0;
    APart = [];
end

end

function mins = local_maxes(x)

x = -x(:);
m = [0 ; (x(1:end-2) > x(2:end-1)) & (x(3:end) > x(2:end-1)) ; 0];
mins = find(m);

end
